==> synapselabels-3.2.0-py3-none-any.whl: synapselabels/svgparse.py <==
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

from yaml import dump
from os.path import join
from os.path import dirname
import cairo
import subprocess
import xml.etree.ElementTree as ET
from .labelconfig import LabelConfig
from .labelconfig import label_config_file

units = ['em', 'ex', 'px', 'pt', 'pc', 'cm', 'mm', 'in', 'percentages']
svg_dir = join(dirname(__file__), 'svg')


def get_mm(value, unit):
    if unit == 'mm':
        return float(value)
    elif unit == 'in':
        return float(value) / 25.4
    raise ValueError('unit %s is not yet supported' % unit)


def get_svg_file(file_name):
    return join(
        svg_dir,
        file_name if file_name.endswith('.svg') else (file_name + '.svg')
        )


def get_extents(text, font):
    surface = cairo.ImageSurface(cairo.FORMAT_A1, 200, 200)
    cr = cairo.Context(surface)
    cr.select_font_face(
        font,
        cairo.FONT_SLANT_NORMAL,
        cairo.FONT_WEIGHT_NORMAL,
        )
    return cr.text_extents(text)


def value_unit(value_and_unit):
    for unit in units:
        if value_and_unit.endswith(unit):
            value = float(value_and_unit[:-len(unit)])
            return value, unit


class TextObject():
    def __init__(self, x, y, r, font, size, text):
        self.x = x
        self.y = y
        self.r = str(r)
        self.font = font
        self.size = size
        self.text = text.replace(u'\u2019', '?')

    def __str__(self):
        return 'T %.3f,%.3f,%s,%s,%.3f;%s' % (
            self.x, self.y, self.r, self.font, self.size, self.text
            )


class ImageObject():
    def __init__(self, x, y, r, name):
        self.x = x
        self.y = y
        self.r = str(r)
        self.name = name

    def __str__(self):
        return 'I %.3f,%.3f,%s;%s' % (
            self.x, self.y, self.r, self.name
            )


class BarcodeObject():
    def __init__(self, x, y, r, barcode):
        self.x = x
        self.y = y
        self.r = str(r)
        self.barcode = barcode

    def __str__(self):
        return 'B %.3f,%.3f,%s,%s' % (
            self.x, self.y, self.r, self.barcode
            )


class RectObject():
    def __init__(self, x, y, r, width, height, thickness):
        self.x = x
        self.y = y
        self.r = str(r)
        self.width = width
        self.height = height
        self.thickness = thickness

    def __str__(self):
        return 'G %.3f,%.3f,%s;R:%.3f,%.3f,%.3f' % (
            self.x, self.y, self.r, self.width, self.height, self.thickness,
            )


def parse_style(style):
    retval = {}
    # print('style:', style)
    for section in style.split(';'):
        if section:
            if ':' in section:
                try:
                    key, value = section.split(':', 1)
                    retval[key] = value
                except Exception:
                    print('section:', repr(section), 'in style:', style)
            else:
                print('wrong style section:', section)
    return retval


def transform_x_y(transform, x_value, y_value):
    x_value = float(x_value)
    y_value = float(y_value)
    if transform.startswith('translate(') and transform.endswith(')'):
        translate = transform[10:-1]
        if ',' in translate:
            translate = translate.split(',')
            return (
                x_value + float(translate[0]),
                y_value + float(translate[1]),
                )
        else:
            return (
                x_value + float(translate),
                y_value,
                )
    elif transform.startswith('matrix(') and transform.endswith(')'):
        a, b, c, d, e, f = (float(x) for x in transform[7: -1].split(','))
        return (
            a * x_value + c * y_value + e,
            b * x_value + d * y_value + f,
            )
    elif transform.startswith('scale(') and transform.endswith(')'):
        scale = transform[6:-1]
        if ',' in scale:
            scale = scale.split(',')
            return (
                x_value * float(scale[0]),
                y_value * float(scale[1]),
                )
        else:
            return (
                x_value * float(scale),
                y_value * float(scale),
                )
    elif transform == 'rotate(-90)':
        return y_value, x_value
    else:
        raise ValueError(transform)


class Transform(list):
    def __init__(self, transform_list):
        super(Transform, self).__init__()
        self.extend(transform_list.split(' '))

    def x_y(self, x_value, y_value):
        for transform in self:
            # print 'transform:', transform
            x_value, y_value = transform_x_y(transform, x_value, y_value)
        return x_value, y_value

    def rotation(self):
        if 'matrix(0,-1,1,0,0,0)' in self:
            return 90
        elif 'matrix(0,1,-1,0,0,0)' in self:
            return -90
        elif 'scale(-1,-1)' in self:
            return 180
        else:
            return 0


def get_font(style):
    """
    {'font-family': "'Swiss 721'", 'font-weight': 'bold'}: 5
    {'font-family': "'Swiss 721'", 'font-weight': 'normal'}: 3
    """
    if style['font-family'] == "'Swiss 721'":
        if style['font-weight'] == 'bold':
            return '5'
        elif style['font-weight'] == 'normal':
            return '3'
        else:
            raise ValueError(
                'Unsupported Swiss 721 font-weight: %s' % style['font-weight']
                )
    elif style['font-family'] == "'Monospace 821'":
        if style['font-weight'] == 'normal':
            return '596'
        else:
            raise ValueError(
                'Unsupported Monospace 821 font-weight: %s' %
                style['font-weight']
                )

    # elif style['font-family'] == "Arial":
    #     if style['font-weight'] == 'bold':
    #         return '11'
    #     elif style['font-weight'] == 'normal':
    #         return '10'
    #     else:
    #         raise ValueError(
    #             'Unsupported Arial font-weight: %s' % style['font-weight']
    #             )
    else:
        raise ValueError(
            'Unsupported font-family: %s' % style['font-family']
            )


class SvgParser(LabelConfig):
    def __init__(self, config_file):
        LabelConfig.__init__(self, config_file)

    def get_label(self, name):
        # print('name:', name)
        if name not in self.svgfiles:
            print('not implemented label type: %s' % name)
            return None
        svg_config_data = self.svgfiles[name]
        svg_file = get_svg_file(name)
        replace = (
            svg_config_data['replace']
            if svg_config_data and 'replace' in svg_config_data else None
            )
        items = []
        # print('parsing:', name)
        tree = ET.parse(svg_file)
        svg = tree.getroot()
        height, height_unit = value_unit(svg.attrib['height'])
        width, width_unit = value_unit(svg.attrib['width'])
        if height_unit == width_unit:
            unit = height_unit
        else:
            raise ValueError('height_unit does not equal width_unit')
        viewBox = [float(i) for i in svg.attrib['viewBox'].split(' ')]
        graphic = svg.findall(".//{http://www.w3.org/2000/svg}g")[0]
        graphic_transform = (
            Transform(graphic.attrib['transform'])
            if 'transform' in graphic.attrib else None
            )
        for aaa in graphic:
            if 'x' in aaa.attrib and 'y' in aaa.attrib:
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                x, y = aaa.attrib['x'], aaa.attrib['y']
                if aaa_transform:
                    x, y = aaa_transform.x_y(x, y)
                if graphic_transform:
                    x, y = graphic_transform.x_y(x, y)
                x = get_mm(x * (width / viewBox[2]), unit)
                y = get_mm(y * (height / viewBox[3]), unit)
            else:
                print("%s is missing x or y" % aaa.tag)
            if aaa.tag == '{http://www.w3.org/2000/svg}text':
                style = parse_style(aaa.attrib['style'])
                for bbb in aaa:
                    if bbb.tag == '{http://www.w3.org/2000/svg}tspan':
                        if 'style' in bbb.attrib:
                            style.update(parse_style(bbb.attrib['style']))
                        text_transform = (
                            Transform(aaa.attrib['transform'])
                            if 'transform' in aaa.attrib else None
                            )
                        text = (
                            replace[bbb.text]
                            if replace and bbb.text in replace else bbb.text
                            )
                        if 'font-size' in style:
                            font_size_mm = get_mm(
                                float(
                                    style['font-size'][:-2]
                                    ) * (height / viewBox[3]),
                                unit
                                )
                        else:
                            raise ValueError(
                                'no "font-size" in style of text id: %s' %
                                aaa.attrib['id']
                                )
                        text_object = TextObject(
                            x,
                            y,
                            text_transform.rotation() if text_transform else 0,
                            get_font(style),
                            font_size_mm,
                            text,
                            )
                        items.append(str(text_object))
            elif aaa.tag == '{http://www.w3.org/2000/svg}image':
                # print('image attrib:', aaa.attrib)
                if '{http://www.w3.org/1999/xlink}href' not in aaa.attrib:
                    raise ValueError('image is not a link')
                image_name = aaa.attrib['{http://www.w3.org/1999/xlink}href']
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                if (
                    'qrcode' in image_name.lower() or
                        'barcode' in image_name.lower()):
                    if image_name not in replace:
                        raise ValueError(
                            "replacement is missing for %s" % image_name
                            )
                    new_object = BarcodeObject(
                        x,
                        y,
                        aaa_transform.rotation() if aaa_transform else 0,
                        replace[image_name],
                        )
                else:
                    if image_name not in self.image_names:
                        raise ValueError(
                            "image name is missing for %s" % image_name
                            )
                    new_object = ImageObject(
                        x,
                        y,
                        aaa_transform.rotation() if aaa_transform else 0,
                        self.image_names[image_name],
                        )
                items.append(str(new_object))
            elif aaa.tag == '{http://www.w3.org/2000/svg}rect':
                style = parse_style(aaa.attrib['style'])
                if 'stroke-width' not in style:
                    raise ValueError(
                        'No stroke-width in id: %s' % aaa.attrib['id']
                        )
                stroke_width = style['stroke-width']
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                rect_object = RectObject(
                    x,
                    y,
                    aaa_transform.rotation() if aaa_transform else 0,
                    get_mm(
                        float(aaa.attrib['width']) * (width / viewBox[2]),
                        unit
                        ),
                    get_mm(
                        float(aaa.attrib['height']) * (height / viewBox[3]),
                        unit
                        ),
                    get_mm(
                        float(stroke_width[:-2]) * (width / viewBox[2]),
                        unit
                        ),
                    )
                items.append(str(rect_object))
            else:
                raise ValueError("Not recognized tag: %s", aaa.tag)
        svg_config_data['items'] = items
        return svg_config_data

    def get_labels(self):
        labels = {}
        for name in self.svgfiles:
            print('get label:', name)
            labels[name] = self.get_label(name)
        return {'labels': labels}

    def get_labels_dump(self):
        return dump(
            self.get_labels(),
            default_flow_style=False,
            width=10000
            )

    def print_barcodes(self):
        for name, svg_config_data in self.svgfiles.items():
            if 'replace' in svg_config_data:
                replace = svg_config_data['replace']
                replace_dict = {}
                for key, value in replace.items():
                    if value.startswith('{') and value.endswith('}'):
                        replace_dict[value[1:-1]] = key
                # print('replace_dict:', replace_dict)
                for key, value in replace.items():
                    if key.endswith('.png'):
                        print(
                            'printing %s to file: %s' % (
                                repr(value),
                                key
                                )
                            )
                        if value.startswith('QRCODE'):
                            qrcode = value.format(**replace_dict)
                            filename = join(dirname(__file__), 'svg', key)
                            print(
                                'printing %s to file: %s' % (
                                    repr(qrcode),
                                    filename
                                    )
                                )
                            print_qrcode(qrcode, filename)


def dots(size, unit='m', dpi=300):
    if size.startswith('pt'):
        size = float(size[2:],) / 72.0
    else:
        size = float(size)
        if unit == 'm':
            size = size / 25.4  # convert mm to inch
        elif unit != 'i':
            raise ValueError(
                'invalid unit: %s' % repr(unit)
                )
    return dpi * size


def print_qrcode(param_string, qr_file_name):
    control, text = param_string.split(';', 1)
    barcode, size = control.split(',', 1)
    barcode_opts = barcode.split('+')
    if barcode_opts[0] != 'QRCODE':
        raise ValueError('Not a QR code: %s' % param_string)
    qr_model = None
    error_correction = None
    for opt in barcode_opts:
        if opt == 'QRCODE':
            pass
        elif opt == 'MODEL1':
            qr_model = 1
        elif opt == 'MODEL2':
            qr_model = 2
        elif opt == 'ELL':
            error_correction = 'L'
        elif opt == 'ELM':
            error_correction = 'M'
        elif opt == 'ELQ':
            error_correction = 'Q'
        elif opt == 'ELH':
            error_correction = 'H'
        else:
            raise ValueError('unsupported qrcode option: %s' % opt)
    box_size = int(0.5 + dots(size))
    if error_correction is None:
        raise ValueError('unspecified error correction')
    if qr_model != 2:
        raise ValueError('unsupported QR Code model: %s' % qr_model)
    args = [
        'qrencode',
        '-o', qr_file_name,
        '-l', error_correction,
        '-m', '0',
        '-s', str(box_size),
        text
        ]
    print("args:", args)
    subprocess.check_call(args)


def svg_parse():
    parser = SvgParser(label_config_file)
    print(parser.get_labels_dump())


def print_barcodes():
    parser = SvgParser(label_config_file)
    print(parser.print_barcodes())

==> synapselabels-3.5.6-py3-none-any.whl: synapselabels/svgparse.py <==
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

from yaml import dump
from os.path import join
from os.path import dirname
import cairo
import subprocess
import xml.etree.ElementTree as ET
from .labelconfig import LabelConfig
from .labelconfig import label_config_file

units = ['em', 'ex', 'px', 'pt', 'pc', 'cm', 'mm', 'in', 'percentages']
svg_dir = join(dirname(__file__), 'svg')


def get_mm(value, unit):
    if unit == 'mm':
        return float(value)
    elif unit == 'in':
        return float(value) / 25.4
    raise ValueError('unit %s is not yet supported' % unit)


def get_svg_file(file_name):
    return join(
        svg_dir,
        file_name if file_name.endswith('.svg') else (file_name + '.svg')
        )


def get_extents(text, font):
    surface = cairo.ImageSurface(cairo.FORMAT_A1, 200, 200)
    cr = cairo.Context(surface)
    cr.select_font_face(
        font,
        cairo.FONT_SLANT_NORMAL,
        cairo.FONT_WEIGHT_NORMAL,
        )
    return cr.text_extents(text)


def value_unit(value_and_unit):
    for unit in units:
        if value_and_unit.endswith(unit):
            value = float(value_and_unit[:-len(unit)])
            return value, unit


class TextObject():
    def __init__(self, x, y, r, font, size, text):
        self.x = x
        self.y = y
        self.r = str(r)
        self.font = font
        self.size = size
        self.text = text.replace(u'\u2019', '?')

    def __str__(self):
        return 'T %.3f,%.3f,%s,%s,%.3f;%s' % (
            self.x, self.y, self.r, self.font, self.size, self.text
            )


class ImageObject():
    def __init__(self, x, y, r, name):
        self.x = x
        self.y = y
        self.r = str(r)
        self.name = name

    def __str__(self):
        return 'I %.3f,%.3f,%s;%s' % (
            self.x, self.y, self.r, self.name
            )


class BarcodeObject():
    def __init__(self, x, y, r, barcode):
        self.x = x
        self.y = y
        self.r = str(r)
        self.barcode = barcode

    def __str__(self):
        return 'B %.3f,%.3f,%s,%s' % (
            self.x, self.y, self.r, self.barcode
            )


class RectObject():
    def __init__(self, x, y, r, width, height, thickness):
        self.x = x
        self.y = y
        self.r = str(r)
        self.width = width
        self.height = height
        self.thickness = thickness

    def __str__(self):
        return 'G %.3f,%.3f,%s;R:%.3f,%.3f,%.3f' % (
            self.x, self.y, self.r, self.width, self.height, self.thickness,
            )


def parse_style(style):
    retval = {}
    # print('style:', style)
    for section in style.split(';'):
        if section:
            if ':' in section:
                try:
                    key, value = section.split(':', 1)
                    retval[key] = value
                except Exception:
                    print('section:', repr(section), 'in style:', style)
            else:
                print('wrong style section:', section)
    return retval


def transform_x_y(transform, x_value, y_value):
    x_value = float(x_value)
    y_value = float(y_value)
    if transform.startswith('translate(') and transform.endswith(')'):
        translate = transform[10:-1]
        if ',' in translate:
            translate = translate.split(',')
            return (
                x_value + float(translate[0]),
                y_value + float(translate[1]),
                )
        else:
            return (
                x_value + float(translate),
                y_value,
                )
    elif transform.startswith('matrix(') and transform.endswith(')'):
        a, b, c, d, e, f = (float(x) for x in transform[7: -1].split(','))
        return (
            a * x_value + c * y_value + e,
            b * x_value + d * y_value + f,
            )
    elif transform.startswith('scale(') and transform.endswith(')'):
        scale = transform[6:-1]
        if ',' in scale:
            scale = scale.split(',')
            return (
                x_value * float(scale[0]),
                y_value * float(scale[1]),
                )
        else:
            return (
                x_value * float(scale),
                y_value * float(scale),
                )
    elif transform == 'rotate(-90)':
        return y_value, x_value
    else:
        raise ValueError(transform)


class Transform(list):
    def __init__(self, transform_list):
        super(Transform, self).__init__()
        self.extend(transform_list.split(' '))

    def x_y(self, x_value, y_value):
        for transform in self:
            # print 'transform:', transform
            x_value, y_value = transform_x_y(transform, x_value, y_value)
        return x_value, y_value

    def rotation(self):
        if 'matrix(0,-1,1,0,0,0)' in self:
            return 90
        elif 'matrix(0,1,-1,0,0,0)' in self:
            return -90
        elif 'scale(-1,-1)' in self:
            return 180
        else:
            return 0


def get_font(style):
    """
    font numbers:
    {'font-family': "'Swiss 721'", 'font-weight': 'bold'}: 5
    {'font-family': "'Swiss 721'", 'font-weight': 'normal'}: 3
    """
    if style['font-family'] == "'Swiss 721'":
        if style['font-weight'] == 'bold':
            return '5'
        elif style['font-weight'] == 'normal':
            return '3'
        else:
            raise ValueError(
                'Unsupported Swiss 721 font-weight: %s' % style['font-weight']
                )
    elif style['font-family'] == "'Monospace 821'":
        if style['font-weight'] == 'normal':
            return '596'
        else:
            raise ValueError(
                'Unsupported Monospace 821 font-weight: %s' %
                style['font-weight']
                )
    # elif style['font-family'] == "Arial":
    #     if style['font-weight'] == 'normal':
    #         return '10'
    #     elif style['font-weight'] == 'bold':
    #         return '11'
    #     else:
    #         raise ValueError(
    #             'Unsupported Arial font-weight: %s' % style['font-weight']
    #             )
    # elif style['font-family'] == "'Trebuchet MS'":
    #     if style['font-weight'] == 'normal':
    #         return '12'
    #     elif style['font-weight'] == 'bold':
    #         return '13'
    #     else:
    #         raise ValueError(
    #             'Unsupported Trebuchet MS font-weight: %s' % style[
    #                 'font-weight'
    #                 ]
    #             )
    else:
        raise ValueError(
            'Unsupported font-family: %s' % style['font-family']
            )


class SvgParser(LabelConfig):
    def __init__(self, config_file):
        LabelConfig.__init__(self, config_file)

    def get_label(self, name):
        # print('name:', name)
        if name not in self.templates:
            print('not implemented label type: %s' % name)
            return None
        svg_config_data = self.templates[name]
        svg_file = get_svg_file(name)
        replace = (
            svg_config_data['replace']
            if svg_config_data and 'replace' in svg_config_data else None
            )
        items = []
        # print('parsing:', name)
        tree = ET.parse(svg_file)
        svg = tree.getroot()
        height, height_unit = value_unit(svg.attrib['height'])
        width, width_unit = value_unit(svg.attrib['width'])
        if height_unit == width_unit:
            unit = height_unit
        else:
            raise ValueError('height_unit does not equal width_unit')
        viewBox = [float(i) for i in svg.attrib['viewBox'].split(' ')]
        graphic = svg.findall(".//{http://www.w3.org/2000/svg}g")[0]
        graphic_transform = (
            Transform(graphic.attrib['transform'])
            if 'transform' in graphic.attrib else None
            )
        for aaa in graphic:
            if 'x' in aaa.attrib and 'y' in aaa.attrib:
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                x, y = aaa.attrib['x'], aaa.attrib['y']
                if aaa_transform:
                    x, y = aaa_transform.x_y(x, y)
                if graphic_transform:
                    x, y = graphic_transform.x_y(x, y)
                x = get_mm(x * (width / viewBox[2]), unit)
                y = get_mm(y * (height / viewBox[3]), unit)
            else:
                print("%s is missing x or y" % aaa.tag)
            if aaa.tag == '{http://www.w3.org/2000/svg}text':
                style = parse_style(aaa.attrib['style'])
                for bbb in aaa:
                    if bbb.tag == '{http://www.w3.org/2000/svg}tspan':
                        if 'style' in bbb.attrib:
                            style.update(parse_style(bbb.attrib['style']))
                        text_transform = (
                            Transform(aaa.attrib['transform'])
                            if 'transform' in aaa.attrib else None
                            )
                        text = (
                            replace[bbb.text]
                            if replace and bbb.text in replace else bbb.text
                            )
                        if 'font-size' in style:
                            font_size_mm = get_mm(
                                float(
                                    style['font-size'][:-2]
                                    ) * (height / viewBox[3]),
                                unit
                                )
                        else:
                            raise ValueError(
                                'no "font-size" in style of text id: %s' %
                                aaa.attrib['id']
                                )
                        text_object = TextObject(
                            x,
                            y,
                            text_transform.rotation() if text_transform else 0,
                            get_font(style),
                            font_size_mm,
                            text,
                            )
                        items.append(str(text_object))
            elif aaa.tag == '{http://www.w3.org/2000/svg}image':
                # print('image attrib:', aaa.attrib)
                if '{http://www.w3.org/1999/xlink}href' not in aaa.attrib:
                    raise ValueError('image is not a link')
                image_name = aaa.attrib['{http://www.w3.org/1999/xlink}href']
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                if (
                    'qrcode' in image_name.lower() or
                        'barcode' in image_name.lower()):
                    if image_name not in replace:
                        raise ValueError(
                            "replacement is missing for %s" % image_name
                            )
                    new_object = BarcodeObject(
                        x,
                        y,
                        aaa_transform.rotation() if aaa_transform else 0,
                        replace[image_name],
                        )
                else:
                    if image_name not in self.image_names:
                        raise ValueError(
                            "image name is missing for %s" % image_name
                            )
                    new_object = ImageObject(
                        x,
                        y,
                        aaa_transform.rotation() if aaa_transform else 0,
                        self.image_names[image_name],
                        )
                items.append(str(new_object))
            elif aaa.tag == '{http://www.w3.org/2000/svg}rect':
                style = parse_style(aaa.attrib['style'])
                if 'stroke-width' not in style:
                    raise ValueError(
                        'No stroke-width in id: %s' % aaa.attrib['id']
                        )
                stroke_width = style['stroke-width']
                aaa_transform = (
                    Transform(aaa.attrib['transform'])
                    if 'transform' in aaa.attrib else None
                    )
                rect_object = RectObject(
                    x,
                    y,
                    aaa_transform.rotation() if aaa_transform else 0,
                    get_mm(
                        float(aaa.attrib['width']) * (width / viewBox[2]),
                        unit
                        ),
                    get_mm(
                        float(aaa.attrib['height']) * (height / viewBox[3]),
                        unit
                        ),
                    get_mm(
                        float(stroke_width[:-2]) * (width / viewBox[2]),
                        unit
                        ),
                    )
                items.append(str(rect_object))
            else:
                raise ValueError("Not recognized tag: %s", aaa.tag)
        svg_config_data['items'] = items
        return svg_config_data

    def get_labels(self):
        labels = {}
        for name in self.templates:
            print('get label:', name)
            labels[name] = self.get_label(name)
        return {'labels': labels}

    def get_labels_dump(self):
        return dump(
            self.get_labels(),
            default_flow_style=False,
            width=10000
            )

    def print_barcodes(self):
        for name, svg_config_data in self.templates.items():
            if 'replace' in svg_config_data:
                replace = svg_config_data['replace']
                replace_dict = {}
                for key, value in replace.items():
                    if value.startswith('{') and value.endswith('}'):
                        replace_dict[value[1:-1]] = key
                # print('replace_dict:', replace_dict)
                for key, value in replace.items():
                    if key.endswith('.png'):
                        print(
                            'printing %s to file: %s' % (
                                repr(value),
                                key
                                )
                            )
                        if value.startswith('QRCODE'):
                            qrcode = value.format(**replace_dict)
                            filename = join(dirname(__file__), 'svg', key)
                            print(
                                'printing %s to file: %s' % (
                                    repr(qrcode),
                                    filename
                                    )
                                )
                            print_qrcode(qrcode, filename)


def dots(size, unit='m', dpi=300):
    if size.startswith('pt'):
        size = float(size[2:],) / 72.0
    else:
        size = float(size)
        if unit == 'm':
            size = size / 25.4  # convert mm to inch
        elif unit != 'i':
            raise ValueError(
                'invalid unit: %s' % repr(unit)
                )
    return dpi * size


def print_qrcode(param_string, qr_file_name):
    control, text = param_string.split(';', 1)
    barcode, size = control.split(',', 1)
    barcode_opts = barcode.split('+')
    if barcode_opts[0] != 'QRCODE':
        raise ValueError('Not a QR code: %s' % param_string)
    qr_model = None
    error_correction = None
    for opt in barcode_opts:
        if opt == 'QRCODE':
            pass
        elif opt == 'MODEL1':
            qr_model = 1
        elif opt == 'MODEL2':
            qr_model = 2
        elif opt == 'ELL':
            error_correction = 'L'
        elif opt == 'ELM':
            error_correction = 'M'
        elif opt == 'ELQ':
            error_correction = 'Q'
        elif opt == 'ELH':
            error_correction = 'H'
        else:
            raise ValueError('unsupported qrcode option: %s' % opt)
    box_size = int(0.5 + dots(size))
    if error_correction is None:
        raise ValueError('unspecified error correction')
    if qr_model != 2:
        raise ValueError('unsupported QR Code model: %s' % qr_model)
    args = [
        'qrencode',
        '-o', qr_file_name,
        '-l', error_correction,
        '-m', '0',
        '-s', str(box_size),
        text
        ]
    print("args:", args)
    subprocess.check_call(args)


def svg_parse():
    parser = SvgParser(label_config_file)
    print(parser.get_labels_dump())


def print_barcodes():
    parser = SvgParser(label_config_file)
    print(parser.print_barcodes())
